import unittest
from mock import Mock, patch

from beeline.middleware.django import HoneyMiddlewareBase

class SimpleWSGITest(unittest.TestCase):
    def setUp(self):
        self.addCleanup(patch.stopall)
        self.m_gbl = patch('beeline.middleware.django.beeline').start()

    def test_call_middleware(self):
        ''' Just call the middleware and ensure that the code runs '''
        mock_req = Mock()
        mock_resp = Mock()
        mock_trace = Mock()
        self.m_gbl.start_trace.return_value = mock_trace

        mw = HoneyMiddlewareBase(mock_resp)
        resp = mw(mock_req)
        self.m_gbl.start_trace.assert_called_once()

        mock_resp.assert_called_once_with(mock_req)

        self.m_gbl.finish_trace.assert_called_once_with(mock_trace)
        self.assertEqual(resp, mock_resp.return_value)


from __future__ import absolute_import
import six
import unittest
from mock import Mock, patch

import urllib

class TestUrllibPatch(unittest.TestCase):
    @unittest.skipIf(six.PY2, "urllib not compatible with python2")
    def test_request_fn_injects_headers_and_returns(self):
        from beeline.patch.urllib import _urllibopen # pylint: disable=bad-option-value,import-outside-toplevel

        with patch('beeline.get_beeline') as m_bl:
            bl = Mock()
            m_bl.return_value = bl

            trace_context = "1;trace_id=foo,parent_id=bar,context=base64value=="

            bl.tracer_impl.marshal_trace_context.return_value = trace_context
            # this is our request call that's being wrapped
            m_urlopen = Mock()
            m_urlopen.return_value = Mock(headers={'content-type': 'application/json', 'content-length': 23}, status_code=500)
            args = ['https://example.com']
            kwargs = {}
            ret = _urllibopen(m_urlopen, None, args, kwargs)

            # ensure our arg gets modified and header set before the real function is called
            self.assertEqual(type(args[0]), urllib.request.Request)
            self.assertEqual(args[0].full_url, 'https://example.com') # pylint: disable=no-member
            self.assertEqual(args[0].headers['X-Honeycomb-Trace'], trace_context) # pylint: disable=no-member
            m_urlopen.assert_called_once_with(*args, **kwargs)
            m_urlopen.reset_mock()
            
            # ensure we return a response
            self.assertEqual(ret, m_urlopen.return_value)

            # test case with Request object
            m_urlopen.return_value = Mock(headers={'content-type': 'application/json', 'content-length': 23}, status_code=500)
            req = urllib.request.Request('https://example.com/2')
            args = [req]
            ret = _urllibopen(m_urlopen, None, args, kwargs)
            self.assertEqual(type(args[0]), urllib.request.Request)
            self.assertEqual(args[0].full_url, 'https://example.com/2')
            self.assertEqual(args[0].headers['X-Honeycomb-Trace'], trace_context)
            self.assertEqual(ret, m_urlopen.return_value)
            m_urlopen.assert_called_once_with(*args, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['beeline',
 'beeline.middleware',
 'beeline.middleware.awslambda',
 'beeline.middleware.bottle',
 'beeline.middleware.django',
 'beeline.middleware.flask',
 'beeline.middleware.werkzeug',
 'beeline.patch']

package_data = \
{'': ['*']}

install_requires = \
['libhoney>=1.7.0', 'wrapt>=1.12.1,<2.0.0']

entry_points = \
{'console_scripts': ['tests = beeline.test_suite:run_tests']}

setup_kwargs = {
    'name': 'honeycomb-beeline',
    'version': '2.12.2.dev1',
    'description': 'Honeycomb library for easy instrumentation',
    'long_description': '# Honeycomb Beeline for Python\n\n[![Build Status](https://circleci.com/gh/honeycombio/beeline-python.svg?style=svg)](https://app.circleci.com/pipelines/github/honeycombio/beeline-python)\n\nThis package makes it easy to instrument your Python web application to send useful events to [Honeycomb](https://honeycomb.io), a service for debugging your software in production.\n\n- [Usage and Examples](https://docs.honeycomb.io/getting-data-in/beelines/beeline-python/)\n- [API Reference](https://honeycombio.github.io/beeline-python/)\n\n## Compatible with\n\nCurrently supports Django (>1.10), Flask, Bottle, and Tornado.\n\nCompatible with both Python 2.7 and Python 3.\n\n## Get in touch\n\nPlease reach out to [support@honeycomb.io](mailto:support@honeycomb.io) or ping\nus with the chat bubble on [our website](https://www.honeycomb.io) for any\nassistance. We also welcome [bug reports](https://github.com/honeycombio/beeline-python/issues).\n\n## Contributions\n\nFeatures, bug fixes and other changes to `beeline-python` are gladly accepted. Please\nopen issues or a pull request with your change. Remember to add your name to the\nCONTRIBUTORS file!\n\nIf you add a new test module, be sure and update `beeline.test_suite` to pick up the new tests.\n\nAll contributions will be released under the Apache License 2.0.\n',
    'author': 'Honeycomb.io',
    'author_email': 'feedback@honeycomb.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/honeycombio/beeline-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=2.7',
}


setup(**setup_kwargs)



import beeline
from beeline.trace import unmarshal_trace_context

# In Lambda, a cold start is when Lambda has to spin up a new instance of a
# function to satisfy a request, rather than re-use an existing instance.
# This usually has a non-trivial effect on latency for the request and is
# worth instrumenting.
COLD_START = True

def _get_trace_data_from_message_attributes(attributes):
    ''' Look for the trace data from SNS/SQS Messsage Atrributes
    '''
    trace_id, parent_id, context = None, None, None

    if isinstance(attributes, dict):
        keymap = {k.lower(): k for k in attributes.keys()}
        if 'x-honeycomb-trace' in keymap:
            if 'Value' in attributes[keymap['x-honeycomb-trace']]:
                # SNS
                trace_id, parent_id, context = unmarshal_trace_context(
                    attributes[keymap['x-honeycomb-trace']]['Value']
                )
            elif 'stringValue' in attributes[keymap['x-honeycomb-trace']]:
                # SQS
                trace_id, parent_id, context = unmarshal_trace_context(
                    attributes[keymap['x-honeycomb-trace']]['stringValue']
                )

    return trace_id, parent_id, context

def _get_trace_data(event):
    ''' Extract trace/parent ids and context object that are threaded through
    in various ways from other beelines'''
    trace_id, parent_id, context = None, None, None

    # Look for trace headers in common places
    if isinstance(event, dict):
        # If API gateway is triggering the Lambda, the event will have headers
        # and we can look for our trace headers
        # https://docs.aws.amazon.com/lambda/latest/dg/with-on-demand-https.html
        if 'headers' in event:
            if isinstance(event['headers'], dict):
                # deal with possible case issues
                keymap = {k.lower(): k  for k in event['headers'].keys()}
                if 'x-honeycomb-trace' in keymap:
                    trace_id, parent_id, context = unmarshal_trace_context(
                        event['headers'][keymap['x-honeycomb-trace']]
                    )

        # If a message source is triggering the Lambda, the event may have
        # our trace data in the message attributes
        elif 'Records' in event:
            # Only process batches of exactly 1
            #  Higher batch sizes would have multiple messages thus
            #  generating multiple traces and requiring manual instrumentation
            if len(event['Records']) == 1:
                # If SNS is triggering the Lambda
                # https://docs.aws.amazon.com/lambda/latest/dg/with-sns.html
                if 'EventSource' in event['Records'][0]:
                    if event['Records'][0]['EventSource'] == 'aws:sns':
                        trace_id, parent_id, context = _get_trace_data_from_message_attributes(
                            event['Records'][0]['Sns']['MessageAttributes']
                        )

                # If SQS is triggering the Lambda
                # https://docs.aws.amazon.com/lambda/latest/dg/with-sqs.html
                elif 'eventSource' in event['Records'][0]:
                    if event['Records'][0]['eventSource'] == 'aws:sqs':
                        trace_id, parent_id, context = _get_trace_data_from_message_attributes(
                            event['Records'][0]['messageAttributes']
                        )

    return trace_id, parent_id, context

def beeline_wrapper(handler):
    ''' Honeycomb Beeline decorator for Lambda functions. Expects a handler
    function with the signature:

    `def handler(event, context)`

    Example use:

    ```
    @beeline_wrapper
    def my_handler(event, context):
        # ...
    ```

    '''

    def _beeline_wrapper(event, context):
        global COLD_START

        # don't blow up the world if the beeline has not been initialized
        if not beeline.get_beeline():
            return handler(event, context)

        try:
            # assume we're going to get bad values sometimes in our headers
            trace_id, parent_id, trace_context = None, None, None
            try:
                trace_id, parent_id, trace_context = _get_trace_data(event)
            except Exception as e:
                beeline.internal.log('error attempting to extract trace context: %s', beeline.internal.stringify_exception(e))
                pass
            with beeline.tracer(name=handler.__name__,
                    trace_id=trace_id, parent_id=parent_id):
                beeline.add_context({
                    "app.function_name": getattr(context, 'function_name', ""),
                    "app.function_version": getattr(context, 'function_version', ""),
                    "app.request_id": getattr(context, 'aws_request_id', ""),
                    "app.event": event,
                    "meta.cold_start": COLD_START,
                })

                # if there is custom context attached from upstream, add that now
                if isinstance(trace_context, dict):
                    for k, v in trace_context.items():
                        beeline.add_trace_field(k, v)

                resp = handler(event, context)

                if resp is not None:
                    beeline.add_context_field('app.response', resp)

                return resp
        finally:
            # This remains false for the lifetime of the module
            COLD_START = False
            # we have to flush events before the lambda returns
            beeline.get_beeline().client.flush()

    return _beeline_wrapper
